<?php
/**
 * Description of Session
 *
 * @author jkraaijeveld
 */

class Session extends ResearchToolObject
{
	public $title;
	public $creator;
	public $creationdate;
	public $pipeline;

	
	/**
	 * Constructor for a Session object
	 * @param type $uid
	 * @param type $title
	 * @param type $creator
	 * @param type $datetime
	 * @param type $pipeline
	 */
	public function __construct($uid = null, $title = null, $creator = null, $datetime = null, $pipeline = null)
	{
		if(!isset($uid))
		{
			$uid = md5(uniqid(rand(), true));
		}
		$this->uid = $uid;
		$this->title = $title;
		$this->creator = $creator;
		$this->creationdate = $datetime;
		$this->pipeline = $pipeline;
	}

	/*
	 * function evaluate(()
	 * Evaluates all the references of this object.
	 */
	public function evaluate()
	{
		$dbi = new DatabaseInterface();
		if(is_string($this->creator))
		{
			$result = $dbi->get("user", array("uid" => $this->creator));
			if(!isset($result[0]))
				return false;
			$this->creator = $result[0];
		}
		if(!empty($this->pipeline) && is_string($this->pipeline[0]))
		{
			$newPipeline = array();
			foreach($this->pipeline as $element)
			{
				//Check if the element's UID can be found in surveys, if not: 
				//Check applications. If it isn't in either: invalid reference.
				$result = $dbi->get("survey", array("uid" => $element));
				if(!isset($result[0]))
					$result = $dbi->get("application", array("uid" => $element));
				if(!isset($result[0]))
					return false;
				$newPipeline[] = $result[0];
			}
			$this->pipeline = $newPipeline;
		}
		return true;
	}

	/**
	 * function toSPSS
	 * @param type $location
	 * Writes an SPSS parseable file at the given location, named '$datetime_$title.txt'
	 * One big note about toSPSS: If this session contains multiple instances of THE SAME Survey,
	 * the world burns, people die, etcetera. (or, in a more fortunate case, the latest values
	 * are written to the file whilst the others get lost.
	 */
	//DO NOT USE - BROKEN AS OF 23/12/2011
	public function toSPSS($location)
	{
		$this->evaluate();
		if(!is_dir($location))
		{
			mkdir($location);
		}
		//Create the filename, open the file at the given location
		$fileName = $this->datetime->format("d-m-Y") . "_" . $this->title . ".txt";

		//Make a list of all 'column headers', e.g. "respondent" and all the question codes.
		$headers = array();
		$headers[] = "respondent";
		foreach ($this->pipeline as $element)
		{
			if(get_class($element) == "Survey")
			{
				$element->evaluate();
				foreach($element->questions as $question)
				{
					//TODO: This might need fixing after lazy initialization refactoring.
					$headers[] = $element->title . '_' . $question->uid;
				}
			}
		}
		//Return if there are no surveys.
		if(count($headers) < 2)
			return;
		
		//Get the answers for all respondents
		$answers = $this->getAnswers($headers);

		//Create the strings and write to file.
		$headerString = "";
		foreach($headers as $headerElement)
		{
			$headerString = $headerString . $headerElement . "\t";
		}
		$fileHandler = fopen($location . '/' . $fileName, 'w') or die("File in use");
		fwrite($fileHandler, $headerString . "\n");
		foreach($answers as $entry)
		{
			$entryString = "";
			//PHP doesn't automatically sort the indices so I have to do it myself
			//Entry elements can now be printed in order to match the headers
			for ($i = 0; $i < count($headers); $i++)
			{
				if(in_array($i, array_keys($entry)))
					$entryString = $entryString . $entry[$i] . "\t";	
				else
					$entryString = $entryString . "\t";
			}
			fwrite($fileHandler, $entryString . "\n");	
		}
	}

	/*
	 * function getAnswers
	 * @param type $headers
	 * Based on the questions specified in the headers array, matches the answers of every
	 * respondent to the correct questions and returns an array containing a list of answers per 
	 * respondent
	 */
	private function getAnswers($headers)
	{
		//Add the answers per survey per respondent. Requires some extra looping to assure proper ordering.
		//Keeps a list of respondents which have already been used. If an answerset exists for a respondent which
		//already exists within the answers-array, use the index of the respondent. Else: make a new row.
		$respondentsUsed = array();
		$answers = array();
		foreach($this->answersets as $answerset)
		{
			$entryIndex;
			if(in_array($answerset->respondent->name, $respondentsUsed))
			{
				$entryIndex = array_search($answerset->respondent->name, $respondentsUsed);
				$entry = $answers[$entryIndex];
			}
			else
			{
				$entry = array();
				$entry[0] = $answerset->respondent->name;
				$entryIndex = count($respondentsUsed);
				$respondentsUsed[] = $answerset->respondent->name;
			}
			foreach ($answerset->answers as $answer)
			{
				$index = array_search($answerset->survey->title . '_' . $answer->question->uid, $headers);
				//Index has to be a good number, else the answer doesn't belong in this session
				//Causes might be a missing question or having a wrong survey
				if($index != 0)
				{
					$value = "";
					foreach ($answer->values as $val)
					{
						$value = $value . $val . ",";
					}
					//We all hate trailing commas
					if (strlen($value) > 0) { $value = substr($value, 0, strlen($value)-1); }
					$entry[$index] = $value;
				}
			}
			$answers[$entryIndex] = $entry;
		}
		return $answers;
	}

}	


?>
